//==============================================================================
//
// Title:		Programmatic Fan Control of PXIe Chassis
// Purpose:		This example demonstrates how to use the System Configuration
//				API to programmatically control the fan mode and fan speed
//				of a supported National Instruments PXI Express chassis.
//
// Software:    NI PXI Platform Services 16.0 or later
//              NI System Configuration 16.0 or later
// Hardware:    NI PXIe-1085, PXIe-1086, or PXIe-1066DC
// Copyright:	National Instruments. All Rights Reserved.
//
//==============================================================================

//==============================================================================
// Include files

#include "nisyscfg.h"
#include "nisyscfg_pxi.h"
#include <ansi_c.h>
#include <cvirte.h>		
#include <userint.h>
#include "PxiFanControlWithGui.h"
#include "toolbox.h"
#include <stdio.h>

//==============================================================================
// Static global variables

static int panelHandle = 0;
static NISysCfgResourceHandle chassis; // Global handle to PXI chassis.

//==============================================================================
// Global functions

int main (int argc, char *argv[])
{
	int error = 0;
	
	NISysCfgStatus status;		   // Stores errors returned by System Configuration functions.
	NISysCfgSessionHandle session; // Handle for System Configuration session.
	
	// Initialize and load resources.
	nullChk (InitCVIRTE (0, argv, 0));
	errChk (panelHandle = LoadPanel (0, "PxiFanControlWithGui.uir", PANEL));
	
	// Open the System Configuration session
    status = NISysCfgInitializeSession(
        NULL,                  // targetName localhost
        NULL,                  // username
        NULL,                  // password
        NISysCfgLocaleDefault, // default language
        NISysCfgBoolTrue,      // forcePropertyRefresh
        1000,                  // connectTimeoutMsec
        NULL,                  // expertEnumHandle
        &session
        );
	
	// Verify that the System Configuration session was opened properly.
	if (status != NISysCfg_OK)
    {
        printf("Error creating session: %d\n", status);
        return 0;
    }
	
	// Create a System Configuration filter to get all PXI chassis in the system.
    NISysCfgFilterHandle filter;
    status = NISysCfgCreateFilter(session, &filter);
    
	// Verify that the filter was created properly.
	if (status != NISysCfg_OK)
    {
    	NISysCfgCloseHandle(session);
    	printf("Error creating filter: %d\n", status);
    	return 0;
    }
	
	// Set filter attribute to filter for PCI/PXI bus type.
	status = NISysCfgSetFilterProperty(filter, NISysCfgFilterPropertyProvidesBusType, NISysCfgBusTypePciPxi);
    
	// Verify that the filter attribute was set properly.
	if (status != NISysCfg_OK)
    {
    	NISysCfgCloseHandle(filter);
    	NISysCfgCloseHandle(session);
    	printf("Error setting filter attribute NISysCfgFilterPropertyProvidesBusType NISysCfgBusTypePciPxi: %d\n", status);
    	return 0;
    }
	
	// Find all PXI chassis in the system.
    NISysCfgEnumResourceHandle chassisHandles;

    status = NISysCfgFindHardware(
        session,
        NISysCfgFilterModeMatchValuesAll,
        filter,
        "ni-pxi",  // Limit to the PXI Platform Services expert.
        &chassisHandles
        );
    
	NISysCfgCloseHandle(filter);
    
	// Verify that filtering for harware completed successfully.
	if (status != NISysCfg_OK)
    {
      NISysCfgCloseHandle(session);
      printf("Error filtering for chassis: %d\n", status);
      return 0;
    }
	
	// Close SysCfg session and notify user if no PXI chassis found in the system.
	if (chassisHandles == NULL)
    {
      NISysCfgCloseHandle(session);
      printf("No PXI chassis in the system.");
      return 0;
    }
	
	// Assign current chassis to chassis global variable.
	status = NISysCfgNextResource(session, chassisHandles, &chassis);							   
	
	// Display the panel and run the user interface.
	errChk (DisplayPanel (panelHandle));
	errChk (RunUserInterface ());
	
Error:
	// Clean up.
	if (panelHandle > 0)
		DiscardPanel (panelHandle);
	return 0;
}

//==============================================================================
// UI callback function prototypes

/// HIFN Exit when the user dismisses the panel.
int CVICALLBACK panelCB (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	if (event == EVENT_CLOSE)
		QuitUserInterface (0);
	return 0;
}

// Returns current fan mode for chassis.
int CVICALLBACK getFanMode (int panel, int control, int event,
							void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			NISysCfgPxiFanModes fanMode; // Stores the chassis's current fan mode.
			NISysCfgStatus status;		 // Stores errors returned by System Configuration functions.
			
			// Get fan mode and assign to variable.
			status = NISysCfgGetResourceProperty(chassis, (NISysCfgResourceProperty)NISysCfgPxiPropertyFanMode, &fanMode);

			// Update UI indicator to display fan mode.
			if (fanMode == 1)
				SetCtrlVal (panel, PANEL_FANMODEOUT, "Fan Mode: Auto");
			else if (fanMode == 2)
				SetCtrlVal (panel, PANEL_FANMODEOUT, "Fan Mode: Safe Manual");
			
			// Verify that fan mode was acquired successfully.
			if (status != NISysCfg_OK)
    		{
      			printf("Error acquiring fan mode: %d\n", status);
      			return 0;
    		}
			
			break;
	}
	return 0;
}

// Returns current fan speed for chassis.
int CVICALLBACK getFanSpeed (int panel, int control, int event,
							 void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			int fanSpeed = 0;				  // Stores the chassis's current fan speed.
			int* fanSpeedPointer = &fanSpeed; // Pointer to fan speed value.
			NISysCfgStatus status;			  // Stores errors returned by System Configuration functions.
			
			// Get fan speed and assign to variable.
			status = NISysCfgGetResourceProperty(chassis, (NISysCfgResourceProperty)NISysCfgPxiPropertyFanUserRpm, fanSpeedPointer);
			fanSpeed = *fanSpeedPointer;
			
			// Update UI indicator to display fan speed.
			SetCtrlVal(panel, PANEL_FANSPEEDOUT, fanSpeed);

			// Verify that fan speed was acquired successfully.
			if (status != NISysCfg_OK)
    		{
      			printf("Error acquiring fan speed: %d\n", status);
      			return 0;
    		}
			
			break;
	}
	return 0;
}

// Assigns fan mode to chassis based on value of control SETFANMODE.
int CVICALLBACK setFanMode (int panel, int control, int event,
							void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			char** detailedResult = NULL;			// Stores return value of System Configuration functions indicating detailed results of operation.
			int fanMode = 0;						// Stores the desired fan mode.
			NISysCfgBool changesRequireRestart = 0; // Stores return value of System Configuration functions indicating whether a system reboot is necessary.
			NISysCfgStatus status;					// Stores errors returned by System Configuration functions.
			
			// Initialize property access by first getting the fan mode once using System Configuration.
			status = NISysCfgGetResourceProperty(chassis, (NISysCfgResourceProperty)NISysCfgPxiPropertyFanMode, &fanMode);
			GetCtrlVal (panel, PANEL_SETFANMODE, &fanMode);
			
			// Assign desired fan mode to chassis and save resource changes.
			if (fanMode == 0)
			{
				status = NISysCfgSetResourceProperty(chassis, (NISysCfgResourceProperty)NISysCfgPxiPropertyFanMode, NISysCfgPxiFanModesAuto);
				status = NISysCfgSaveResourceChanges(chassis, &changesRequireRestart, detailedResult);
			}
			else if (fanMode == 1)
			{
				status = NISysCfgSetResourceProperty(chassis, (NISysCfgResourceProperty)NISysCfgPxiPropertyFanMode, NISysCfgPxiFanModesSafeManual);
				status = NISysCfgSaveResourceChanges(chassis, &changesRequireRestart, detailedResult);
			}
			else
			{				
				// Attempting to set invalid fan mode for chassis. Return error to user.
				printf("Error assigning fan mode: %d\n", status);
      			return 0;
			}
			
			// Verify that fan mode was assigned successfully.
			if (status != NISysCfg_OK)
    		{
      			printf("Error assigning fan mode: %d\n", status);
      			return 0;
    		}
			
			break;
	}
	return 0;
}

// Assigns fan speed to chassis based on value of control SETFANSPEED.
int CVICALLBACK setFanSpeed (int panel, int control, int event,
						   void *callbackData, int eventData1, int eventData2)
{
	switch (event)
	{
		case EVENT_COMMIT:
			char** detailedResult = NULL;			// Stores return value of System Configuration functions indicating detailed results of operation.
			const unsigned long fanSpeed = 2200;	// Stores the desired fan speed.
			NISysCfgBool changesRequireRestart = 0; // Stores return value of System Configuration functions indicating whether a system reboot is necessary.
			NISysCfgStatus status = 0;				// Stores errors returned by System Configuration functions.
			
			// Initialize property access by first getting the fan speed once using System Configuration.
			status = NISysCfgGetResourceProperty(chassis, (NISysCfgResourceProperty)NISysCfgPxiPropertyFanUserRpm, &fanSpeed);
			GetCtrlVal(panel, PANEL_FANSPEED, &fanSpeed);
			
			// Update UI indicator to display fan speed to user.
			SetCtrlVal(panel, PANEL_FANSPEEDOUT, fanSpeed);
			
			// Assign desired fan speed to chassis and save resource changes.
			status = NISysCfgSetResourceProperty(chassis, (NISysCfgResourceProperty)NISysCfgPxiPropertyFanUserRpm, fanSpeed);
			status = NISysCfgSaveResourceChanges(chassis, &changesRequireRestart, detailedResult);
			
			// Verify that fan speed was assigned successfully.
			if (status != NISysCfg_OK)
    		{
      			printf("Error assigning fan speed: %d\n", status);
      			return 0;
    		}
			
			break;
	}
	return 0;
}
